# -*- coding: utf-8 -*-
"""
main working functions file
"""

import requests, re
from time import time
from transfer.request import MakeRequest
from transfer.exceptions import FileTooLarge
from random import choices
from string import digits, ascii_uppercase, ascii_lowercase

class Upload(MakeRequest):
	'''Upload file

	:param str file: name of the file of full location
	:param **kwargs: Optional arguments that :class:`MakeRequest <MakeRequest>` class takes.
	'''
	def __init__(self, file:str, verbose:bool=False, cli:bool=False,**kwargs):
		super().__init__(file=file, **kwargs)

		start = time()
		self.verbose = verbose
		if self.verbose:
			print("Preparing upload file...")

		self.prepare()

		response = self._send(**self.kwargs)

		assert response.status_code == 200, response.reason

		self.uploading_process_time = time() - start

		self._link = self.process(response)

		if cli:
			self._cli()

	def _send(self, **kwargs) -> requests.Response:
		'''make put request to upload file

		:return: :class:`requests.Response <requests.Response>` object
		:rtype: requests.Response
		'''
		return requests.put(**kwargs)

	def prepare(self):
		response = requests.get(self.url, headers={"accept": "text/html"})
		assert response.status_code == 200, response.reason

		pattern = re.compile(r"<h3>Upload up to ([0-9]+) (GB|MB)</h3>")

		size = {
			"MB": 1024*1024,
			"GB": 1024*1024*1024
		}
		nsize, type = re.findall(pattern, response.text)[0]

		self._max_file_limit = int(nsize)*size[type]

		if self._max_file_limit < self._len:
			FileTooLarge(self._max_file_limit)

	def _gen_hash(self) -> str:
		return "".join([
				*choices(ascii_uppercase+ascii_lowercase, k=2),
				*choices(digits, k=4),
				*choices(ascii_uppercase+ascii_lowercase, k=2)
				])


	def process(self, response:requests.Response) -> str:
		self.data = {
			"file": self._file_name,
			"hash": self._gen_hash(),
			"created on": response.headers.get("Date"),
			"delete url": response.headers.get("X-Url-Delete"),
			"url": response.text
		}
		return response.text

	def _cli(self) -> None:
		if self.verbose:
			print(
				f"Uploading time: {self.uploading_time:.2f}", 
				f"Server processing time: {(self.uploading_process_time - self.uploading_time):.2f}",
				sep=" | "
			)
		print(f"Link: {self._link}")
		if self.verbose:
			print(f"Hash: {self.data.get('hash')}")
			print(f"Delete: {self.data.get('delete url')}")
			print(f"Created on: {self.data.get('created on')}")

	@property
	def link(self):
		return self._link
	


class Remove:
	def __init__(self, *args, **kwargs):
		response = self._send(*args, **kwargs)
		if response.status_code == 200:
			self.status = True
			self.reason = "Successful delete" 
		elif response.status_code == 404:
			self.status = True
			self.reason = "Not found, already deleted"
		else:
			self.status = False
			self.reason = response.reason


	def _send(self, *args, **kwargs) -> requests.Response:
		'''make delete request to delete uploaded file

		:return: :class:`requests.Response <requests.Response>` object
		:rtype: requests.Response
		'''
		return requests.delete(*args, **kwargs)


'''
A simple table generator

file: https://gist.github.com/MayankFawkes/9c26882cea0fce086eeac689c418a0fa
'''

class Table:
	def __init__(self, titles:tuple, margin:int=0):
		self.titles = [n.upper() for n in titles]
		self.margin = margin
		self.width = len(titles)
		self.len = [len(title) for title in self.titles]
		self.tables = list()

	def _display(self):
		self.len = [n+self.margin for n in self.len]
		self.tables = [self.titles, *self.tables]
		tiles = [f"{'-'*n}" for n in self.len]
		tiles = "+".join(tiles)
		print(f"+{tiles}+")
		for n in range(len(self.tables)):
			r_display = list()
			for l, name in zip(self.len, self.tables[n]):
				r_display.append(f"{name or str('NA'):^{l}}")
			r_display = "|".join(r_display)
			print(f"|{r_display}|")
			print(f"+{tiles}+")


	def _mod_len(self, entries:tuple):
		count = 0 
		for past, new in zip(self.len, map(len, entries)):
			if past < new:
				self.len[count] = new
			count+=1

	def add_table(self, *entries:tuple):
		if len(entries) < len(self.titles):
			entries = [*entries, *["" for n in range(len(self.titles) - len(entries))]]
		self._mod_len(entries)
		self.tables.append(entries)

	def display(self):
		self._display()


# -*- coding: utf-8 -*-
"""
This is unofficial CLI of transfer.sh
"""

__author__ = "Mayank Gupta"
__license__ = "MIT License"
__github__ = "https://github.com/MayankFawkes/transfer.sh"

from transfer.request import MakeRequest
from transfer.__main__ import Upload, Remove
from transfer.version import __version__


# -*- coding: utf-8 -*-
"""
Cli file
"""
import sys

if __name__ == '__main__':
	sys.path.insert(0,'..')


from transfer import Upload, Remove, __version__, __github__
from transfer.table import Table
import argparse, re, signal
from os import access, R_OK
from os.path import isfile
from typing import Dict
from typing import Union
from typing import List
from json import loads, dumps


class ActionCheck(object):
	AVAIABLE_TEST = ["file", "hash"]
	def __init__(self, test:str):
		self.test = test
		assert test in self.AVAIABLE_TEST, "Invalid Test"
		self._hash_pattern = re.compile(r"([a-zA-Z]){2}([0-9]){4}([a-zA-Z]){2}")

	def match_hash(self, value:str) -> bool:
		if re.match(self._hash_pattern, value):
			return True
		return False

	def file(self, value:str) -> Dict[str, str]:
		if isfile(value) and access(value, R_OK):
			return dict(type="file", value=value)
		raise argparse.ArgumentTypeError(f"can't open '{value}' No such file or do not have permissions")

	def hash(self, value:str) -> Dict[str, str]:
		if self.match_hash(value) and len(value) == 8:
			return dict(type="hash", value=value)
		raise argparse.ArgumentTypeError(f"Invalid hash")

	def __call__(self, value):
		return getattr(self, self.test)(value)

class log:
	FILE_NAME = "transfer.log"
	def __init__(self):
		if not isfile(self.FILE_NAME):
			open(self.FILE_NAME, 'a').close()

	def _get(self) -> Union[List[None], List[Dict[str, str]]]:
		with open(self.FILE_NAME, "r") as file:
			data = file.read()
		file.close()
		if data:
			return loads(data)
		return list()

	def _save(self, data:List[Dict[str, str]]=list()) -> None:
		with open(self.FILE_NAME, "w") as file:
			file.write(dumps(data))
		file.close()

	def _filter(self, data:List[Dict[str, str]], hash:str) -> Union[Dict[str, str], None]:
		for log in data:
			if log.get("hash") == hash:
				return log
		raise KeyError(f"Hash not found")

	def _add_log(self, data:Dict[str, str]):
		log = self._get()
		log.append(data)
		self._save(log)

	def _remove_log_with_hash(self, log:Dict[str, str]):
		logs = self._get()
		logs.remove(log)
		self._save(logs)


class CLI(log):
	def __init__(self):
		signal.signal(signal.SIGINT, self.bye)
		signal.signal(signal.SIGTERM, self.bye)

		super().__init__()

		args = self.get_args()

		if args.type.get("type") == "file":
			self._upload(args)

		elif args.type.get("type") == "list":
			self._list()

		elif args.type.get("type") == "hash":
			self._remove(args)

	def _upload(self, namespace:argparse.Namespace) -> None:
		namespace = vars(namespace)
		namespace.__setitem__("file", namespace.pop("type").get("value"))
		res = Upload(cli=True,**namespace)
		self._add_log(res.data)

	def _remove(self, namespace:argparse.Namespace):
		try:
			namespace = vars(namespace)
			namespace.__setitem__("hash", namespace.pop("type").get("value"))

			logs = self._get()

			log = self._filter(**namespace, data=logs)

			response = Remove(url=log.get("delete url"))

			if response.status:
				self._remove_log_with_hash(log)
				print(f"Deleted: {response.reason}")

			else:
				print(f"Error: {response.reason}")


		except KeyError:
			print(f"Error: Invalid hash, Not found.")

	def _list(self):
		logs = self._get()

		table = Table(["file", "hash", "created", "url"], margin=5)

		for log in logs:
			table.add_table(log.get("file"), log.get("hash"), log.get("created on"), log.get("url"))

		table.display()



	def get_args(self) -> argparse.Namespace:
		parser = argparse.ArgumentParser(description="transfer.sh CLI")

		parser.add_argument("-V", "--version", action="version", version=f"transfer.sh version {__version__} ({__github__})")
		sub_parser = parser.add_subparsers(
							help="Help", 
							metavar="Options", 
							required=True,
							title="Required Options"
							)

		c1 = sub_parser.add_parser("remove", aliases=["rm"], help="Remove uploaded file.")
		c1.add_argument("type", type=ActionCheck("hash"), help="Hash of already uploaded file.")

		c2 = sub_parser.add_parser("upload", aliases=["up"], help="Upload file.")
		c2.add_argument("type", type=ActionCheck("file"), help="File you want to upload.")

		c2.add_argument("-v", "--verbose", required=False, action="store_true", help="Enable Verbose mode for extra logs.")
		c2.add_argument("-np", "--no-process-bar", required=False, action="store_true", help="Disable process bar.")
		c2.add_argument("-s", "--url", required=False, help="Third-party transfer.sh servers.")
		
		c3 = sub_parser.add_parser("list", aliases=["l"], help="List all uploaded files.")
		c3.add_argument("-s", "--show", action="store_const", required=True, const=dict(type="list"), dest="type")
		return parser.parse_args()

	def bye(self, signum, frame):
		print("Quiting...")
		sys.exit(0)



def main():
	CLI()

if __name__ == "__main__":
	main()


import pathlib
from setuptools import setup
from transfer import __version__

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="transfer.sh",
    version=f"{__version__}",
    description="transfer.sh CLI",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/MayankFawkes/transfer.sh",
    author="Mayank Gupta",
    author_email="mayankfawkes@gmail.com",
    license="MIT License",
    project_urls={
            "Documentation": "https://github.com/MayankFawkes/transfer.sh/blob/master/README.md",
            "Source": "https://github.com/MayankFawkes/transfer.sh",
        },
    classifiers=[
        "Topic :: Software Development :: Libraries :: Python Modules",
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3',
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Development Status :: 4 - Beta",
        'Topic :: System :: Networking',
        'Topic :: Internet :: WWW/HTTP',
        "Natural Language :: English",
        'Environment :: Plugins',
        "Environment :: Console",
        "Topic :: Terminals",
        "Topic :: Utilities",
        ],
    entry_points={
        "console_scripts": ["transfer = transfer.cli:main"],
        },
    packages=["transfer"],
    python_requires=">=3.6",
    keywords=["upload", "cli", "transfer.sh"],
    include_package_data=True,
    install_requires=["requests"],
)



# -*- coding: utf-8 -*-
"""
Exceptions classes
"""


class TransferError(Exception):
	def __init__(self, msg):
		self.msg = msg
		Exception.__init__(self, msg)

	def __str__(self):
		return self.msg

	__repr__ = __str__


class FileTooLarge(TransferError):
	"""Base video unavailable error."""
	def __init__(self, limit: int):
		"""
		:param int limit:
			Max uploading size.
		"""
		self._limit = limit
		super().__init__(self.error_message)

	@property
	def error_message(self):
		return f'Max file size limit is {self.limit}'

	@property
	def limit(self):
		return str(round(self._limit/(1024*1024*1024), 2))+'GB' if len(str(self._limit//(1024*1024))) >= 4 else str(round(self._limit/(1024*1024), 2))+'MB'



# -*- coding: utf-8 -*-
"""
Helper classes file
"""

from mimetypes import guess_type
from time import time
from os.path import getsize, split
from datetime import timedelta

from typing import Generator
from typing import Dict
from typing import Union

class MakeRequest:
	'''Construct arguments to make request.

	:param str file: name of the file of full location
	:param object callback: this will get execute when ``self._run_callback`` is ``True``, default is ``False``
	:param dict cb_kwargs: additional keyword arguments for backback 

	'''

	def __init__(self, 
			file:str, 
			url:str=None, 
			callback:object=None,
			no_process_bar:bool=True,
			cb_kwargs:dict=dict(end="\r")
		):
		self._len = getsize(file)
		self._content_type, encoding = guess_type(file)

		self._file = open(file, "rb")
		tail, self._file_name = split(file)

		self.url = url or "https://transfer.sh"
		self._upload_url = f"{self.url}/{self._file_name}" if self.url[-1] != "/" else f"{self.url}{self._file_name}"

		self._progress = 0
		self._progress_in_second = 0

		self._callback = callback or self._progress_bar
		self._run_callback = not no_process_bar
		self._cb_kwargs = cb_kwargs
		self._started = self._last_callback = time()
		self._uploading_time = None

	def __len__(self):
		return self._len

	def iter_content(self, chunk_size:int=8192) -> Generator[int, bytes, None]:
		''' Generator so data cant be in memory in case of large files.

		:param int chunk_size: chunk size 

    	:rtype: collections.Iterable
		'''
		while data:=self._file.read(chunk_size):
			self._cb_kwargs.update({
				'size'	: self._len,
				'progress': self._progress,
				'in_second': self._progress_in_second
			})

			if self._run_callback and (time() - self._last_callback) >= 1.0:
				self._callback(**self._cb_kwargs)
				self._last_callback = time()
				self._progress_in_second = 0

			self._progress+=len(data)
			self._progress_in_second+=len(data)
			yield data

		self._cb_kwargs.update({
			'size'	: self._len,
			'progress': self._len,
			'in_second': self._progress_in_second,
			'end': '\n'
		})
		if self._run_callback:
			self._callback(**self._cb_kwargs)

		self._finally()

	def _progress_bar(self, 
			size=None, 
			progress=None,
			in_second=None, 
			loading_sign:str="█", 
			width:int=50, 
			end="\r"
		) -> None:
		percentage = (progress*width) // size
		print(f"Upload: |{loading_sign * percentage:{'-'}<{width}}| {percentage*(100//width):>3}%",
			f"{str(round(in_second/(1024*1024), 2))+'MB/s' if len(str(in_second//1024)) >= 4 else str(in_second//1024)+'Kb/s':>4}",
			f"EST: {self._est(size, progress, in_second):<15}",
			end=end, sep=' | ')

	def _est(self, size, progress, in_second) -> str:
		progress = progress or 1
		in_second = in_second or 1
		time = size//in_second - progress//in_second
		return str(timedelta(seconds=time))

	def _finally(self) -> None:
		self._uploading_time = time() - self._started

	@property
	def kwargs(self) -> Dict[str, Union[str, Generator[int, bytes, None], Dict[str, str]]]:
		return {
			"url": self._upload_url,
			"headers": self.headers,
			"data": self.iter_content()
		}
	
	@property
	def headers(self) -> Dict[str, str]:
		return {
			"Content-Type": self._content_type
		}

	@property
	def uploading_time(self) -> float:
		return self._uploading_time